
jmp main

; ============
; Data section

LoadingMsg db "Preparing to load operating system...", 0x0D, 0x0A, 0
msg_name_ver db "TBOS version 0.2.1", 0x0D, 0x0A, 0

%define		VIDMEM	0xB8000			; video memory
%define		COLS	80			; width and height of screen
%define		LINES	25
%define		CHAR_ATTRIB 63			; character attribute (White text on light blue background)

_CurX db 0					; current x/y location
_CurY db 0

; ============
; System calls

; ---------
; A20 stuff

enable_a20:

	cli
	push ax
	mov	al, 0xdd
	out	0x64, al
	pop	ax
	ret

; ------------------
; Global Desc. Table

gdt_install:

	cli				; clear interrupts
	pusha			; save registers
	lgdt [toc]		; load GDT into GDTR
	sti				; enable interrupts
	popa			; restore registers
	ret				; All done!

gdt_data: 
	dd 0			; null descriptor
	dd 0

					; code descriptor
	dw 0FFFFh		; limit low
	dw 0			; base low
	db 0			; base middle
	db 10011010b	; access
	db 11001111b	; granularity
	db 0			; base high

					; data descriptor
	dw 0FFFFh		; limit low
	dw 0			; base low
	db 0			; base middle
	db 10010010b	; access
	db 11001111b	; granularity
	db 0			; base high
	
end_of_gdt:
toc: 
	dw end_of_gdt - gdt_data - 1	; limit (Size of GDT)
	dd gdt_data						; base of GDT

; give the descriptor offsets names

%define NULL_DESC 0
%define CODE_DESC 0x8
%define DATA_DESC 0x10

; ----------------------
; 16-bit string printing
; Only used before pmode

puts16:
		pusha
.loop:
		lodsb
		or al, al
		jz .done
		mov	ah, 0eh
		int	10h
		jmp	.loop
.done:
		popa
		ret


%include "k_stdio.asm"

; ==================================
; Second stage bootloader and kernel
;
; - zero out all segments
; - install a GDT
; - enable the A20 gate
; - go to protected mode
; - do kernel stuff

main:

	;-------------------------------;
	;   Setup segments and stack	;
	;-------------------------------;

	cli				; clear interrupts
	xor	ax, ax			; null segments
	mov	ds, ax
	mov	es, ax
	mov	ax, 0x9000		; stack begins at 0x9000-0xffff
	mov	ss, ax
	mov	sp, 0xFFFF
	sti				; enable interrupts

	;-------------------------------;
	;   Print loading message	;
	;-------------------------------;

	mov	si, LoadingMsg
	call	puts16

	call	gdt_install

	call	enable_a20

	cli
	mov	eax, cr0
	or	eax, 1
	mov	cr0, eax		; now we're officially in pmode!

	jmp	CODE_DESC:kernel	; far jump to fix CS.

	; From here on out, interrupts should never be re-enabled.
	; If that happens, the CPU will triple fault, resetting the machine.
	; And please, don't use that as a method of resetting casually.

;ENTRY POINT FOR KERNEL

bits 32

kernel:

	; Set us up some registers

	mov		ax, DATA_DESC		; set data segments to data selector (0x10)
	mov		ds, ax
	mov		ss, ax
	mov		es, ax
	mov		esp, 90000h		; stack begins from 90000h

	mov ebx, msg_name_ver
	call puts32
	
	call read_kbd

	cli
	hlt