
putch32:
	; bl=character to print

	pusha
	mov	edi, VIDMEM

	; Get current position

	xor	eax, eax


		mov	ecx, COLS*2		; Mode 7 has 2 bytes per char, so its COLS*2 bytes per line
		mov	al, byte [_CurY]	; get y pos
		mul	ecx			; multiply y*COLS
		push	eax			; save eax--the multiplication

		mov	al, byte [_CurX]	; multiply _CurX by 2 because it is 2 bytes per char
		mov	cl, 2
		mul	cl
		pop	ecx			; pop y*COLS result
		add	eax, ecx

		xor	ecx, ecx
		add	edi, eax		; add it to the base address


	cmp	bl, 0x0A		; is it a newline character?
	je .row			; yep--go to next row

	; Print a character

	mov	dl, bl			; Get character
	mov	dh, CHAR_ATTRIB		; the character attribute
	mov	word [edi], dx		; write to video display

	; Update next position

	inc	byte [_CurX]		; go to next character
	jmp	.done			; nope, bail out

	; Go to next row

.row:
	mov	byte [_CurX], 0		; go back to col 0
	inc	byte [_CurY]		; go to next row

.done:
	popa				; restore registers and return
	ret

; ========================================

puts32:	; ebx=location of zero-terminated string

	; Store registers

	pusha				; save registers
	push	ebx			; copy the string address
	pop	edi

.loop:

	mov	bl, byte [edi]		; get next character
	cmp	bl, 0			; is it 0 (Null terminator)?
	je	.done			; yep-bail out

	call putch32			; Nope-print it out

	inc	edi			; go to next character
	jmp	.loop

.done:

	; VGA is slow, so update the cursor at the end.

	mov	bh, byte [_CurY]	; get current position
	mov	bl, byte [_CurX]
	call	move_cursor		; update cursor

	popa				; restore registers, and return
	ret

; ========================================

move_cursor:	;bl=X, bh=Y

	pusha				; save registers

	; Here, _CurX and _CurY are relitave to the current position on screen, not in memory.
	; That is, we don't need to worry about the byte alignment we do when displaying characters,
	; so just follow the forumla: location = _CurX + _CurY * COLS

	xor	eax, eax
	mov	ecx, COLS
	mov	al, bh			; get y pos
	mul	ecx			; multiply y*COLS
	add	al, bl			; Now add x
	mov	ebx, eax


	; Set low byte index to VGA register

	mov	al, 0x0f
	mov	dx, 0x03D4
	out	dx, al

	mov	al, bl
	mov	dx, 0x03D5
	out	dx, al			; low byte

	; Set high byte index to VGA register

	xor	eax, eax

	mov	al, 0x0e
	mov	dx, 0x03D4
	out	dx, al

	mov	al, bh
	mov	dx, 0x03D5
	out	dx, al			; high byte

	popa
	ret

; ========================================

cls32:

	pusha
	cld
	mov	edi, VIDMEM
	mov	cx, 2000
	mov	ah, CHAR_ATTRIB
	mov	al, ' '	
	rep	stosw

	mov	byte [_CurX], 0
	mov	byte [_CurY], 0
	popa
	ret

; ========================================

setcoords32:		; al=X, ah=Y

	pusha
	mov	[_CurX], al		; just set the current position
	mov	[_CurY], ah
	popa
	ret
	
; ========================================

; ----------------------------------
; Keyboard stuff gets included here.

%include "k_keyb.asm"	; this is the keyboard driver